@@ -5,7 +5,8 @@
 
 logit <- function(
   ### Transforming (0,1) to normal scale (-Inf Inf)
-  p,...
+  p      ##<< percentile
+  ,...   ##<< further arguments to qlogis
 ){ 
   ##details<<
   ## function \eqn{logit(p) = log \left( \frac{p}{1-p} \right) = log(p) - log(1-p)}
@@ -16,7 +17,8 @@ logit <- function(
 
 invlogit <- function(
   ### Transforming (-Inf,Inf) to original scale (0,1)
-  q,...
+  q,   ##<< quantile
+  ...  ##<< further arguments to plogis
 ){
   ##details<<
   ## function \eqn{f(z) = \frac{e^{z}}{e^{z} + 1} \! = \frac{1}{1 + e^{-z}} \!}
@@ -42,8 +44,9 @@ rlogitnorm <- function(
 plogitnorm <- function(
   ### Distribution function for logitnormal distribution	
   q  ##<< vector of quantiles
-  , mu = 0, sigma = 1	##<< distribution parameters
-  , ...
+  , mu = 0    ##<< location distribution parameter
+  , sigma = 1	##<< scale distribution parameter
+  , ...       ##<< further arguments to pnorm
 ){
   ##seealso<< \code{\link{logitnorm}}
   ql <- qlogis(q)
@@ -54,7 +57,8 @@ plogitnorm <- function(
 dlogitnorm <- function(
   ### Density function of logitnormal distribution	
   x		##<< vector of quantiles
-  , mu = 0, sigma = 1	##<< distribution parameters
+  , mu = 0         ##<< scale distribution parameter
+  , sigma = 1	     ##<< location distribution parameter
   , log = FALSE    ##<< if TRUE, the log-density is returned
   , ...	##<< further arguments passed to \code{\link{dnorm}}: \code{mean}, 
   ## and \code{sd} for mu and sigma respectively.  
@@ -89,14 +93,17 @@ dlogitnorm <- function(
 qlogitnorm <- function(
   ### Quantiles of logitnormal distribution.	
   p ##<< vector of probabilities
-  , mu = 0, sigma = 1	##<< distribution parameters
-  , ...
+  , mu = 0    ##<< location distribution parameter
+  , sigma = 1	##<< scale distribution parameter
+  , ...       ##<< further arguments to plogis
 ){
   ##seealso<< \code{\link{logitnorm}}
   qn <- qnorm(p,mean = mu,sd = sigma,...)
   plogis(qn)
 }
 
+# derivative of logit(x)
+.dlogit <- function(x) 1/x + 1/(1-x)
 
 #------------------ estimating parameters from fitting to distribution 
 .ofLogitnorm <- function(
@@ -164,7 +171,6 @@ twCoefLogitnorm <- function(
   median					##<< numeric vector: the median of the density function
   , quant					##<< numeric vector: the upper quantile value
   , perc = 0.975				##<< numeric vector: the probability for which the quantile was specified
-  , ... 
 ){
   ##seealso<< \code{\link{logitnorm}}
   # twCoefLogitnorm
@@ -307,8 +313,6 @@ twCoefLogitnormMLE <- function(
 attr(twCoefLogitnormMLE,"ex") <- function(){
   # estimate the parameters, with mode 0.7 and upper quantile 0.9
   mode = 0.7; upper = 0.9
-  mode = 0.2; upper = 0.7
-  #mode = 0.5; upper = 0.9
   (theta <- twCoefLogitnormMLE(mode,upper))
   x <- seq(0,1,length.out = 41)[-c(1,41)]	# plotting grid
   px <- plogitnorm(x,mu = theta[1],sigma = theta[2])	#percentiles function
@@ -321,18 +325,56 @@ attr(twCoefLogitnormMLE,"ex") <- function(){
   (theta <- twCoefLogitnormMLEFlat(mode))
 }
 
-twCoefLogitnormMLEFlat <- function(
-  ### Estimating coefficients of a maximally flat unimodal logitnormal distribution from mode 	
-  mle						##<< numeric vector: the mode of the density function
-){
-  ##details<<
-  ## When increasing the sigma parameter, the distribution becomes
-  ## eventually becomes bi-model, i.e. has two maxima.	
-  ## This function estimates parameters for given mode, so that the distribution assigns high  
-  ## density to a maximum range, i.e. is maximally flat, but still is unimodal.
-  twCoefLogitnormMLE(mle,0)
+.tmp.f <- function(){
+  (theta = twCoefLogitnormMLEFlat(0.9))
+  (theta = twCoefLogitnormMLEFlat(0.1))
+  (theta = twCoefLogitnormMLEFlat(0.5))
+  mle = c(0.7,0.2)
+  mle = seq(0.55,0.95,by=0.05)
+  mle = seq(0.95,0.99,by=0.01)
+  theta = twCoefLogitnormMLEFlat(mle)
+  sigma = theta[,2]
+  plot(sigma ~ mle)
+  (sigmac = 1/(2*mle*(1-mle)))
+  sigmac/sigma
+  plot(sigmac ~ mle)
 }
 
+twCoefLogitnormMLEFlat <- function(
+  ### Estimating coefficients of a maximally flat unimodal logitnormal distribution given the mode 	
+  mle		##<< numeric vector: the mode of the density function
+){
+  vectorList <- lapply(mle, .twCoefLogitnormMLEFlat_scalar)
+  do.call(rbind, vectorList)
+}
+  
+  
+.twCoefLogitnormMLEFlat_scalar <- function(mle){
+  if (mle == 0.5) {
+    #sigma =  sqrt(dlogit(mle)/2) # 1.414214
+    return(c(mu = 0.0, sigma = sqrt(2)))
+  } 
+  is_right <- mle > 0.5
+  mler = ifelse(is_right, mle, 1-mle)
+  xt <- optimize(.ofModeFlat, interval = c(0,0.5), m=mler, logitm=logit(mler))$minimum
+  sigma2 <- (1/xt + 1/(1-xt))/2
+  mur <- logit(mler) - sigma2*(2*mler - 1)
+  mu <- ifelse(is_right, mur, -mur)
+  c(mu = mu, sigma = sqrt(sigma2))
+}
+
+.ofModeFlat <- function(x, m, logitm = logit(m)){
+  # lhs = 1/x + 1/(1-x)
+  # rhs = (logitm - logit(x))/(m-x)
+  # lhs = (m-x)/x + (m-x)/(1-x)
+  # rhs = (logitm - logit(x))
+  lhs = m/x + (m-1)/(1-x)
+  rhs = logitm - logit(x)
+  d = lhs - rhs
+  d*d
+}
+
+
 .ofLogitnormE <- function(
   ### Objective function used by \code{\link{coefLogitnormE}}. 
   theta				##<< theta[1] is the mu, theta[2] is the sigma
@@ -359,7 +401,7 @@ twCoefLogitnormE <- function(
   , theta0 = c(mu = 0,sigma = 1)	##<< starting parameters
   , returnDetails = FALSE	##<< if TRUE, the full output of optim is returned 
   ## with attribute resOptim
-  , ... 
+  , ...                   ##<< further arguments to optim
 ){
   ##seealso<< \code{\link{logitnorm}}
   # twCoefLogitnormE
